@@ -214,22 +214,12 @@ public class MpscProxyChannelBenchmark {
     public static class ProcessorCounters {
         public long processed;
         public long processFailed;
-
-        @Setup(Level.Iteration)
-        public void clean() {
-            this.processed = this.processFailed = 0;
-        }
     }
 
     @AuxCounters
     @State(Scope.Thread)
     public static class CallerCounters {
         public long callsFailed;
-
-        @Setup(Level.Iteration)
-        public void clean() {
-            this.callsFailed = 0;
-        }
     }
 
     public static final class StoppedException extends RuntimeException {
