@@ -36,16 +36,19 @@ import org.openjdk.jmh.infra.Blackhole;
 @State(Scope.Group)
 @BenchmarkMode(Mode.Throughput)
 @OutputTimeUnit(TimeUnit.MICROSECONDS)
-@Warmup(iterations = 5, time = 3, timeUnit = TimeUnit.SECONDS)
-@Measurement(iterations = 5, time = 3, timeUnit = TimeUnit.SECONDS)
+@Warmup(iterations = 10, time = 1)
+@Measurement(iterations = 10, time = 1)
 public class MpqThroughputBackoffNone {
     private static final long DELAY_PRODUCER = Long.getLong("delay.p", 0L);
     private static final long DELAY_CONSUMER = Long.getLong("delay.c", 0L);
-    Integer ONE = 777;
+    static final Object TEST_ELEMENT = 1;
+    Integer element = 1;
+    Integer escape;
     MessagePassingQueue<Integer> q;
-    private Integer escape;
+
     @Param(value = { "SpscArrayQueue", "MpscArrayQueue", "SpmcArrayQueue", "MpmcArrayQueue" })
     String qType;
+
     @Param(value = { "132000" })
     int qCapacity;
 
@@ -55,23 +58,23 @@ public class MpqThroughputBackoffNone {
 
         // stretch the queue to the limit, working through resizing and full
         for (int i = 0; i < 128 + 100; i++) {
-            q.offer(ONE);
+            q.offer(element);
         }
         for (int i = 0; i < 128 + 100; i++) {
             q.poll();
         }
         // stretch the queue to the limit, working through resizing and full
         for (int i = 0; i < 128 + 100; i++) {
-            q.relaxedOffer(ONE);
+            q.relaxedOffer(element);
         }
         for (int i = 0; i < 128 + 100; i++) {
             q.relaxedPoll();
         }
         // make sure the important common case is exercised
         for (int i = 0; i < 20000; i++) {
-            q.offer(ONE);
+            q.offer(element);
             q.poll();
-            q.relaxedOffer(ONE);
+            q.relaxedOffer(element);
             q.relaxedPoll();
         }
         q = MessagePassingQueueByTypeFactory.createQueue(qType, qCapacity);
@@ -80,40 +83,21 @@ public class MpqThroughputBackoffNone {
     @AuxCounters
     @State(Scope.Thread)
     public static class PollCounters {
-        public int pollsFailed;
-        public int pollsMade;
-
-        @Setup(Level.Iteration)
-        public void clean() {
-            pollsFailed = pollsMade = 0;
-        }
+        public long pollsFailed;
+        public long pollsMade;
     }
 
     @AuxCounters
     @State(Scope.Thread)
     public static class OfferCounters {
-        public int offersFailed;
-        public int offersMade;
-
-        @Setup(Level.Iteration)
-        public void clean() {
-            offersFailed = offersMade = 0;
-        }
-    }
-
-    private static ThreadLocal<Object> marker = new ThreadLocal<Object>();
-
-    @State(Scope.Thread)
-    public static class ConsumerMarker {
-        public ConsumerMarker() {
-            marker.set(this);
-        }
+        public long offersFailed;
+        public long offersMade;
     }
 
     @Benchmark
     @Group("nor")
     public void offerNoR(OfferCounters counters) {
-        if (!q.offer(ONE)) {
+        if (!q.offer(element)) {
             counters.offersFailed++;
             backoff();
         } else {
@@ -126,12 +110,12 @@ public class MpqThroughputBackoffNone {
 
     @Benchmark
     @Group("nor")
-    public void pollNoR(PollCounters counters, ConsumerMarker cm) {
+    public void pollNoR(PollCounters counters) {
         Integer e = q.poll();
         if (e == null) {
             counters.pollsFailed++;
             backoff();
-        } else if (e == ONE) {
+        } else if (e == TEST_ELEMENT) {
             counters.pollsMade++;
         } else {
             escape = e;
@@ -144,7 +128,7 @@ public class MpqThroughputBackoffNone {
     @Benchmark
     @Group("bothr")
     public void offerBothR(OfferCounters counters) {
-        if (!q.relaxedOffer(ONE)) {
+        if (!q.relaxedOffer(element)) {
             counters.offersFailed++;
             backoff();
         } else {
@@ -157,12 +141,12 @@ public class MpqThroughputBackoffNone {
 
     @Benchmark
     @Group("bothr")
-    public void pollBothR(PollCounters counters, ConsumerMarker cm) {
+    public void pollBothR(PollCounters counters) {
         Integer e = q.relaxedPoll();
         if (e == null) {
             counters.pollsFailed++;
             backoff();
-        } else if (e == ONE) {
+        } else if (e == TEST_ELEMENT) {
             counters.pollsMade++;
         } else {
             escape = e;
@@ -175,7 +159,7 @@ public class MpqThroughputBackoffNone {
     @Benchmark
     @Group("pr")
     public void offerPR(OfferCounters counters) {
-        if (!q.relaxedOffer(ONE)) {
+        if (!q.relaxedOffer(element)) {
             counters.offersFailed++;
             backoff();
         } else {
@@ -188,12 +172,12 @@ public class MpqThroughputBackoffNone {
 
     @Benchmark
     @Group("pr")
-    public void pollPR(PollCounters counters, ConsumerMarker cm) {
+    public void pollPR(PollCounters counters) {
         Integer e = q.poll();
         if (e == null) {
             counters.pollsFailed++;
             backoff();
-        } else if (e == ONE) {
+        } else if (e == TEST_ELEMENT) {
             counters.pollsMade++;
         } else {
             escape = e;
@@ -206,7 +190,7 @@ public class MpqThroughputBackoffNone {
     @Benchmark
     @Group("cr")
     public void offerCR(OfferCounters counters) {
-        if (!q.offer(ONE)) {
+        if (!q.offer(element)) {
             counters.offersFailed++;
             backoff();
         } else {
@@ -219,12 +203,12 @@ public class MpqThroughputBackoffNone {
 
     @Benchmark
     @Group("cr")
-    public void pollCR(PollCounters counters, ConsumerMarker cm) {
+    public void pollCR(PollCounters counters) {
         Integer e = q.relaxedPoll();
         if (e == null) {
             counters.pollsFailed++;
             backoff();
-        } else if (e == ONE) {
+        } else if (e == TEST_ELEMENT) {
             counters.pollsMade++;
         } else {
             escape = e;
@@ -236,13 +220,11 @@ public class MpqThroughputBackoffNone {
 
     @TearDown(Level.Iteration)
     public void emptyQ() {
-        if (marker.get() == null)
-            return;
-        // sadly the iteration tear down is performed from each participating
-        // thread, so we need to guess
-        // which is which (can't have concurrent access to poll).
-        while (q.poll() != null)
-            ;
+        synchronized (q)
+        {
+            while (q.poll() != null)
+                ;
+        }
     }
 
     protected void backoff() {
