@@ -60,11 +60,11 @@ public class QueueByTypeFactory {
         case 62:
             return new MpscOnSpscQueue<T>(queueCapacity);
         case 63:
-            return new MpscLinkedQueue8<T>();
+            return new MpscLinkedQueue<T>();
         case 7:
             return new MpmcArrayQueue<T>(queueCapacity);
-            case 708:
-                return BlockingQueueFactory.newBlockingQueue(ConcurrentQueueSpec.createBoundedMpmc(queueCapacity));
+        case 708:
+            return BlockingQueueFactory.newBlockingQueue(ConcurrentQueueSpec.createBoundedMpmc(queueCapacity));
         case 71:
             return new MpmcConcurrentQueueStateMarkers<T>(queueCapacity);
         }
@@ -98,8 +98,10 @@ public class QueueByTypeFactory {
     @SuppressWarnings({ "rawtypes", "unchecked" })
     public static <T> Queue<T> createQueue(String queueType, final int chunkSize, final int queueCapacity) {
         Class qClass = queueClass(queueType);
-        if (!(BaseMpscLinkedArrayQueue.class.isAssignableFrom(qClass) || BaseSpscLinkedArrayQueue.class.isAssignableFrom(qClass)) )
-            throw new IllegalArgumentException("Failed to construct queue, does not expect chunkSize:"+qClass.getName());
+        if (!(BaseMpscLinkedArrayQueue.class.isAssignableFrom(qClass) ||
+              BaseSpscLinkedArrayQueue.class.isAssignableFrom(qClass) ||
+              MpUnboundedXaddArrayQueue.class.isAssignableFrom(qClass)) )
+            throw new IllegalArgumentException("Failed to construct queue, does not expect dual param constructor: " + qClass.getName());
         Constructor constructor;
         Exception ex;
         try {
@@ -117,7 +119,10 @@ public class QueueByTypeFactory {
             return Class.forName("org.jctools.queues."+queueType);
         } catch (ClassNotFoundException e) {
         }
-        
+        try {
+            return Class.forName("org.jctools.queues.atomic."+queueType);
+        } catch (ClassNotFoundException e) {
+        }
         try {
             return Class.forName("java.util."+queueType);
         } catch (ClassNotFoundException e) {
