@@ -12,17 +12,19 @@
  * limitations under the License.
  */
 package org.jctools.maps;
-import static org.jctools.util.UnsafeAccess.UNSAFE;
 
 import java.io.IOException;
 import java.io.Serializable;
-import java.lang.reflect.Field;
 import java.util.*;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.atomic.AtomicLongFieldUpdater;
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
+
 import org.jctools.util.RangeUtil;
 
+import static org.jctools.util.UnsafeAccess.UNSAFE;
+import static org.jctools.util.UnsafeAccess.fieldOffset;
+
 /**
  * A lock-free alternate implementation of {@link java.util.concurrent.ConcurrentHashMap}
  * with better scaling properties and generally lower costs to mutate the Map.
@@ -98,13 +100,8 @@ public class NonBlockingHashMap<TypeK, TypeV>
   }
 
   // --- Setup to use Unsafe
-  private static final long _kvs_offset;
-  static {                      // <clinit>
-    Field f = null;
-    try { f = NonBlockingHashMap.class.getDeclaredField("_kvs"); }
-    catch( java.lang.NoSuchFieldException e ) { throw new RuntimeException(e); }
-    _kvs_offset = UNSAFE.objectFieldOffset(f);
-  }
+  private static final long _kvs_offset = fieldOffset(NonBlockingHashMap.class, "_kvs");
+
   private final boolean CAS_kvs( final Object[] oldkvs, final Object[] newkvs ) {
     return UNSAFE.compareAndSwapObject(this, _kvs_offset, oldkvs, newkvs );
   }
@@ -128,8 +125,6 @@ public class NonBlockingHashMap<TypeK, TypeV>
     return h;
   }
 
- 
-
   // --- The Hash Table --------------------
   // Slot 0 is always used for a 'CHM' entry below to hold the interesting
   // bits of the hash table.  Slot 1 holds full hashes as an array of ints.
@@ -284,17 +279,17 @@ public class NonBlockingHashMap<TypeK, TypeV>
 
   /** Returns the number of key-value mappings in this map.
    *  @return the number of key-value mappings in this map */
-  @Override 
+  @Override
   public int     size       ( )                       { return chm(_kvs).size(); }
   /** Returns <tt>size() == 0</tt>.
    *  @return <tt>size() == 0</tt> */
-  @Override 
+  @Override
   public boolean isEmpty    ( )                       { return size() == 0;      }
 
   /** Tests if the key in the table using the <tt>equals</tt> method.
    * @return <tt>true</tt> if the key is in the table using the <tt>equals</tt> method
    * @throws NullPointerException if the specified key is null  */
-  @Override 
+  @Override
   public boolean containsKey( Object key )            { return get(key) != null; }
 
   /** Legacy method testing if some key maps into the specified value in this
@@ -325,6 +320,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
    *  @return the previous value associated with the specified key,
    *         or <tt>null</tt> if there was no mapping for the key
    *  @throws NullPointerException if the specified key or value is null  */
+  @Override
   public TypeV   putIfAbsent( TypeK  key, TypeV val ) { return putIfMatch( key,      val, TOMBSTONE   ); }
 
   /** Removes the key (and its corresponding value) from this map.
@@ -338,20 +334,26 @@ public class NonBlockingHashMap<TypeK, TypeV>
   /** Atomically do a {@link #remove(Object)} if-and-only-if the key is mapped
    *  to a value which is <code>equals</code> to the given value.
    *  @throws NullPointerException if the specified key or value is null */
-  public boolean remove     ( Object key,Object val ) { return putIfMatch( key,TOMBSTONE, val ) == val; }
+  public boolean remove     ( Object key,Object val ) {
+    return objectsEquals(putIfMatch( key,TOMBSTONE, val ), val);
+  }
 
   /** Atomically do a <code>put(key,val)</code> if-and-only-if the key is
    *  mapped to some value already.
    *  @throws NullPointerException if the specified key or value is null */
+  @Override
   public TypeV   replace    ( TypeK  key, TypeV val ) { return putIfMatch( key,      val,MATCH_ANY   ); }
 
   /** Atomically do a <code>put(key,newValue)</code> if-and-only-if the key is
    *  mapped a value which is <code>equals</code> to <code>oldValue</code>.
    *  @throws NullPointerException if the specified key or value is null */
+  @Override
   public boolean replace    ( TypeK  key, TypeV  oldValue, TypeV newValue ) {
-    return putIfMatch( key, newValue, oldValue ) == oldValue;
+    return objectsEquals(putIfMatch( key, newValue, oldValue ), oldValue);
+  }
+  private static boolean objectsEquals(Object a, Object b) {
+    return (a == b) || (a != null && a.equals(b));
   }
-
 
   // Atomically replace newVal for oldVal, returning the value that existed
   // there before.  If the oldVal matches the returned value, then newVal was
@@ -360,7 +362,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
   public final TypeV putIfMatchAllowNull( Object key, Object newVal, Object oldVal ) {
     if( oldVal == null ) oldVal = TOMBSTONE;
     if( newVal == null ) newVal = TOMBSTONE;
-    final TypeV res = (TypeV)putIfMatch( this, _kvs, key, newVal, oldVal );
+    final TypeV res = (TypeV) putIfMatch0(this, _kvs, key, newVal, oldVal );
     assert !(res instanceof Prime);
     //assert res != null;
     return res == TOMBSTONE ? null : res;
@@ -375,7 +377,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
    */
   public final TypeV putIfMatch( Object key, Object newVal, Object oldVal ) {
     if (oldVal == null || newVal == null) throw new NullPointerException();
-    final Object res = putIfMatch( this, _kvs, key, newVal, oldVal );
+    final Object res = putIfMatch0(this, _kvs, key, newVal, oldVal );
     assert !(res instanceof Prime);
     assert res != null;
     return res == TOMBSTONE ? null : (TypeV)res;
@@ -622,14 +624,31 @@ public class NonBlockingHashMap<TypeK, TypeV>
     }
   }
 
-  // --- putIfMatch ---------------------------------------------------------
-  // Put, Remove, PutIfAbsent, etc.  Return the old value.  If the returned
-  // value is equal to expVal (or expVal is NO_MATCH_OLD) then the put can be
-  // assumed to work (although might have been immediately overwritten).  Only
-  // the path through copy_slot passes in an expected value of null, and
-  // putIfMatch only returns a null if passed in an expected null.
   static volatile int DUMMY_VOLATILE;
-  private static final Object putIfMatch( final NonBlockingHashMap topmap, final Object[] kvs, final Object key, final Object putval, final Object expVal ) {
+  /**
+   * Put, Remove, PutIfAbsent, etc.  Return the old value.  If the returned value is equal to expVal (or expVal is
+   * {@link #NO_MATCH_OLD}) then the put can be assumed to work (although might have been immediately overwritten).
+   * Only the path through copy_slot passes in an expected value of null, and putIfMatch only returns a null if passed
+   * in an expected null.
+   *
+   * @param topmap the map to act on
+   * @param kvs the KV table snapshot we act on
+   * @param key not null (will result in {@link NullPointerException})
+   * @param putval the new value to use. Not null. {@link #TOMBSTONE} will result in deleting the entry.
+   * @param expVal expected old value. Can be null. {@link #NO_MATCH_OLD} for an unconditional put/remove.
+   *              {@link #TOMBSTONE} if we expect old entry to not exist(null/{@link #TOMBSTONE} value).
+   *              {@link #MATCH_ANY} will ignore the current value, but only if an entry exists. A null expVal is used
+   *               internally to perform a strict insert-if-never-been-seen-before operation.
+   * @return {@link #TOMBSTONE} if key does not exist or match has failed. null if expVal is
+   * null AND old value was null. Otherwise the old entry value (not null).
+   */
+  private static final Object putIfMatch0(
+      final NonBlockingHashMap topmap,
+      final Object[] kvs,
+      final Object key,
+      final Object putval,
+      final Object expVal)
+  {
     assert putval != null;
     assert !(putval instanceof Prime);
     assert !(expVal instanceof Prime);
@@ -650,8 +669,8 @@ public class NonBlockingHashMap<TypeK, TypeV>
       if( K == null ) {         // Slot is free?
         // Found an empty Key slot - which means this Key has never been in
         // this table.  No need to put a Tombstone - the Key is not here!
-        if( putval == TOMBSTONE ) return putval; // Not-now & never-been in this table
-        if( expVal == MATCH_ANY ) return null;   // Will not match, even after K inserts
+        if( putval == TOMBSTONE ) return TOMBSTONE; // Not-now & never-been in this table
+        if( expVal == MATCH_ANY ) return TOMBSTONE; // Will not match, even after K inserts
         // Claim the null key-slot
         if( CAS_key(kvs,idx, null, key ) ) { // Claim slot for Key
           chm._slots.add(1);      // Raise key-slots-used count
@@ -699,78 +718,95 @@ public class NonBlockingHashMap<TypeK, TypeV>
         // to claim a key slot (indeed, we cannot find a free one to claim!).
         newkvs = chm.resize(topmap,kvs);
         if( expVal != null ) topmap.help_copy(newkvs); // help along an existing copy
-        return putIfMatch(topmap,newkvs,key,putval,expVal);
+        return putIfMatch0(topmap, newkvs, key, putval, expVal);
       }
 
       idx = (idx+1)&(len-1); // Reprobe!
     } // End of spinning till we get a Key slot
 
-    // ---
-    // Found the proper Key slot, now update the matching Value slot.  We
-    // never put a null, so Value slots monotonically move from null to
-    // not-null (deleted Values use Tombstone).  Thus if 'V' is null we
-    // fail this fast cutout and fall into the check for table-full.
-    if( putval == V ) return V; // Fast cutout for no-change
+    while ( true ) {              // Spin till we insert a value
+      // ---
+      // Found the proper Key slot, now update the matching Value slot.  We
+      // never put a null, so Value slots monotonically move from null to
+      // not-null (deleted Values use Tombstone).  Thus if 'V' is null we
+      // fail this fast cutout and fall into the check for table-full.
+      if( putval == V ) return V; // Fast cutout for no-change
 
-    // See if we want to move to a new table (to avoid high average re-probe
-    // counts).  We only check on the initial set of a Value from null to
-    // not-null (i.e., once per key-insert).  Of course we got a 'free' check
-    // of newkvs once per key-compare (not really free, but paid-for by the
-    // time we get here).
-    if( newkvs == null &&       // New table-copy already spotted?
-        // Once per fresh key-insert check the hard way
-        ((V == null && chm.tableFull(reprobe_cnt,len)) ||
-         // Or we found a Prime, but the JMM allowed reordering such that we
-         // did not spot the new table (very rare race here: the writing
-         // thread did a CAS of _newkvs then a store of a Prime.  This thread
-         // reads the Prime, then reads _newkvs - but the read of Prime was so
-         // delayed (or the read of _newkvs was so accelerated) that they
-         // swapped and we still read a null _newkvs.  The resize call below
-         // will do a CAS on _newkvs forcing the read.
-         V instanceof Prime) )
-      newkvs = chm.resize(topmap,kvs); // Force the new table copy to start
-    // See if we are moving to a new table.
-    // If so, copy our slot and retry in the new table.
-    if( newkvs != null )
-      return putIfMatch(topmap,chm.copy_slot_and_check(topmap,kvs,idx,expVal),key,putval,expVal);
-
-    // ---
-    // We are finally prepared to update the existing table
-    assert !(V instanceof Prime);
-
-    // Must match old, and we do not?  Then bail out now.  Note that either V
-    // or expVal might be TOMBSTONE.  Also V can be null, if we've never
-    // inserted a value before.  expVal can be null if we are called from
-    // copy_slot.
-
-    if( expVal != NO_MATCH_OLD && // Do we care about expected-Value at all?
-        V != expVal &&            // No instant match already?
-        (expVal != MATCH_ANY || V == TOMBSTONE || V == null) &&
-        !(V==null && expVal == TOMBSTONE) &&    // Match on null/TOMBSTONE combo
-        (expVal == null || !expVal.equals(V)) ) // Expensive equals check at the last
-      return V;                                 // Do not update!
-
-    // Actually change the Value in the Key,Value pair
-    if( CAS_val(kvs, idx, V, putval ) ) {
-      // CAS succeeded - we did the update!
-      // Both normal put's and table-copy calls putIfMatch, but table-copy
-      // does not (effectively) increase the number of live k/v pairs.
-      if( expVal != null ) {
-        // Adjust sizes - a striped counter
-        if(  (V == null || V == TOMBSTONE) && putval != TOMBSTONE ) chm._size.add( 1);
-        if( !(V == null || V == TOMBSTONE) && putval == TOMBSTONE ) chm._size.add(-1);
+      // See if we want to move to a new table (to avoid high average re-probe
+      // counts).  We only check on the initial set of a Value from null to
+      // not-null (i.e., once per key-insert).  Of course we got a 'free' check
+      // of newkvs once per key-compare (not really free, but paid-for by the
+      // time we get here).
+      if( newkvs == null &&       // New table-copy already spotted?
+          // Once per fresh key-insert check the hard way
+          ((V == null && chm.tableFull(reprobe_cnt,len)) ||
+           // Or we found a Prime, but the JMM allowed reordering such that we
+           // did not spot the new table (very rare race here: the writing
+           // thread did a CAS of _newkvs then a store of a Prime.  This thread
+           // reads the Prime, then reads _newkvs - but the read of Prime was so
+           // delayed (or the read of _newkvs was so accelerated) that they
+           // swapped and we still read a null _newkvs.  The resize call below
+           // will do a CAS on _newkvs forcing the read.
+           V instanceof Prime) ) {
+        newkvs = chm.resize(topmap, kvs); // Force the new table copy to start
       }
-    } else {                    // Else CAS failed
+      // See if we are moving to a new table.
+      // If so, copy our slot and retry in the new table.
+      if( newkvs != null ) {
+        return putIfMatch0(topmap, chm.copy_slot_and_check(topmap, kvs, idx, expVal), key, putval, expVal);
+      }
+      // ---
+      // We are finally prepared to update the existing table
+      assert !(V instanceof Prime);
+
+      // Must match old, and we do not?  Then bail out now.  Note that either V
+      // or expVal might be TOMBSTONE.  Also V can be null, if we've never
+      // inserted a value before.  expVal can be null if we are called from
+      // copy_slot.
+      if( expVal != NO_MATCH_OLD && // Do we care about expected-Value at all?
+          V != expVal &&            // No instant match already?
+          (expVal != MATCH_ANY || V == TOMBSTONE || V == null) &&
+          !(V==null && expVal == TOMBSTONE) &&    // Match on null/TOMBSTONE combo
+          (expVal == null || !expVal.equals(V)) ) { // Expensive equals check at the last
+        return (V == null) ? TOMBSTONE : V;         // Do not update!
+      }
+
+      // Actually change the Value in the Key,Value pair
+      if( CAS_val(kvs, idx, V, putval ) ) break;
+
+      // CAS failed
+      // Because we have no witness, we do not know why it failed.
+      // Indeed, by the time we look again the value under test might have flipped
+      // a thousand times and now be the expected value (despite the CAS failing).
+      // Check for the never-succeed condition of a Prime value and jump to any
+      // nested table, or else just re-run.
+
+      // We would not need this load at all if CAS returned the value on which
+      // the CAS failed (AKA witness). The new CAS semantics are supported via
+      // VarHandle in JDK9.
       V = val(kvs,idx);         // Get new value
+
       // If a Prime'd value got installed, we need to re-run the put on the
       // new table.  Otherwise we lost the CAS to another racing put.
-      // Simply retry from the start.
       if( V instanceof Prime )
-        return putIfMatch(topmap,chm.copy_slot_and_check(topmap,kvs,idx,expVal),key,putval,expVal);
+        return putIfMatch0(topmap, chm.copy_slot_and_check(topmap, kvs, idx, expVal), key, putval, expVal);
+
+      // Simply retry from the start.
+      // NOTE: need the fence, since otherwise 'val(kvs,idx)' load could be hoisted
+      // out of loop.
+      int dummy = DUMMY_VOLATILE;
     }
-    // Win or lose the CAS, we are done.  If we won then we know the update
-    // happened as expected.  If we lost, it means "we won but another thread
-    // immediately stomped our update with no chance of a reader reading".
+
+    // CAS succeeded - we did the update!
+    // Both normal put's and table-copy calls putIfMatch, but table-copy
+    // does not (effectively) increase the number of live k/v pairs.
+    if( expVal != null ) {
+      // Adjust sizes - a striped counter
+      if(  (V == null || V == TOMBSTONE) && putval != TOMBSTONE ) chm._size.add( 1);
+      if( !(V == null || V == TOMBSTONE) && putval == TOMBSTONE ) chm._size.add(-1);
+    }
+
+    // We won; we know the update happened as expected.
     return (V==null && expVal!=null) ? TOMBSTONE : V;
   }
 
@@ -819,7 +855,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
     // to get the required memory orderings.  It monotonically transits from
     // null to set (once).
     volatile Object[] _newkvs;
-    private final AtomicReferenceFieldUpdater<CHM,Object[]> _newkvsUpdater =
+    private static final AtomicReferenceFieldUpdater<CHM,Object[]> _newkvsUpdater =
       AtomicReferenceFieldUpdater.newUpdater(CHM.class,Object[].class, "_newkvs");
     // Set the _next field if we can.
     boolean CAS_newkvs( Object[] newkvs ) {
@@ -1152,7 +1188,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
       // appears in the old table.
       Object old_unboxed = ((Prime)oldval)._V;
       assert old_unboxed != TOMBSTONE;
-      putIfMatch(topmap, newkvs, key, old_unboxed, null);
+      putIfMatch0(topmap, newkvs, key, old_unboxed, null);
 
       // ---
       // Finally, now that any old value is exposed in the new table, we can
@@ -1217,12 +1253,20 @@ public class NonBlockingHashMap<TypeK, TypeV>
       }                         // Else keep scanning
       return _prevV;            // Return current value.
     }
-    public void remove() {
+
+    public void removeKey() {
       if( _prevV == null ) throw new IllegalStateException();
-      putIfMatch( NonBlockingHashMap.this, _sskvs, _prevK, TOMBSTONE, _prevV );
+      NonBlockingHashMap.this.putIfMatch(_prevK, TOMBSTONE, NO_MATCH_OLD);
       _prevV = null;
     }
 
+    @Override
+    public void remove() {
+      // NOTE: it would seem logical that value removal will semantically mean removing the matching value for the
+      // mapping <k,v>, but the JDK always removes by key, even when the value has changed.
+      removeKey();
+    }
+
     public TypeV nextElement() { return next(); }
     public boolean hasMoreElements() { return hasNext(); }
   }
@@ -1261,7 +1305,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
   private class SnapshotK implements Iterator<TypeK>, Enumeration<TypeK> {
     final SnapshotV _ss;
     public SnapshotK() { _ss = new SnapshotV(); }
-    public void remove() { _ss.remove(); }
+    public void remove() { _ss.removeKey(); }
     public TypeK next() { _ss.next(); return (TypeK)_ss._prevK; }
     public boolean hasNext() { return _ss.hasNext(); }
     public TypeK nextElement() { return next(); }
@@ -1341,7 +1385,11 @@ public class NonBlockingHashMap<TypeK, TypeV>
   private class SnapshotE implements Iterator<Map.Entry<TypeK,TypeV>> {
     final SnapshotV _ss;
     public SnapshotE() { _ss = new SnapshotV(); }
-    public void remove() { _ss.remove(); }
+    public void remove() {
+      // NOTE: it would seem logical that entry removal will semantically mean removing the matching pair <k,v>, but
+      // the JDK always removes by key, even when the value has changed.
+      _ss.removeKey();
+    }
     public Map.Entry<TypeK,TypeV> next() { _ss.next(); return new NBHMEntry((TypeK)_ss._prevK,_ss._prevV); }
     public boolean hasNext() { return _ss.hasNext(); }
   }
@@ -1364,8 +1412,8 @@ public class NonBlockingHashMap<TypeK, TypeV>
    *  requires the creation of {@link java.util.Map.Entry} objects with each
    *  iteration.  The {@link NonBlockingHashMap} does not normally create or
    *  using {@link java.util.Map.Entry} objects so they will be created soley
-   *  to support this iteration.  Iterating using {@link #keySet} or {@link
-   *  #values} will be more efficient.
+   *  to support this iteration.  Iterating using {@link Map#keySet} or {@link
+   *  Map#values} will be more efficient.
    */
   @Override
   public Set<Map.Entry<TypeK,TypeV>> entrySet() {
@@ -1381,7 +1429,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
         if( !(o instanceof Map.Entry)) return false;
         final Map.Entry<?,?> e = (Map.Entry<?,?>)o;
         TypeV v = get(e.getKey());
-        return v.equals(e.getValue());
+        return v != null && v.equals(e.getValue());
       }
       @Override public Iterator<Map.Entry<TypeK,TypeV>> iterator() { return new SnapshotE(); }
     };
@@ -1401,7 +1449,7 @@ public class NonBlockingHashMap<TypeK, TypeV>
   }
 
   // --- readObject --------------------------------------------------------
-  // Read a CHM from a stream
+  // Read a NBHM from a stream
   private void readObject(java.io.ObjectInputStream s) throws IOException, ClassNotFoundException {
     s.defaultReadObject();      // Read nothing
     initialize(MIN_SIZE);
