@@ -12,27 +12,19 @@
  * limitations under the License.
  */
 package org.jctools.maps;
-import static org.jctools.util.UnsafeAccess.UNSAFE;
+
+import org.jctools.util.RangeUtil;
 
 import java.io.IOException;
 import java.io.Serializable;
-import java.lang.reflect.Field;
-import java.util.AbstractCollection;
-import java.util.AbstractMap;
-import java.util.AbstractSet;
-import java.util.Collection;
-import java.util.ConcurrentModificationException;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.Hashtable;
-import java.util.Iterator;
-import java.util.Map;
-import java.util.NoSuchElementException;
-import java.util.Set;
+import java.util.*;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.atomic.AtomicLongFieldUpdater;
 import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
-import org.jctools.util.RangeUtil;
+
+import static org.jctools.maps.NonBlockingHashMap.DUMMY_VOLATILE;
+import static org.jctools.util.UnsafeAccess.UNSAFE;
+import static org.jctools.util.UnsafeAccess.fieldOffset;
 
 
 /**
@@ -75,7 +67,7 @@ import org.jctools.util.RangeUtil;
  * <em>not</em> throw {@link ConcurrentModificationException}.  However,
  * iterators are designed to be used by only one thread at a time.
  *
- * <p> Very full tables, or tables with high reprobe rates may trigger an
+ * <p> Very full tables, or tables with high re-probe rates may trigger an
  * internal resize operation to move into a larger table.  Resizing is not
  * terribly expensive, but it is not free either; during resize operations
  * table throughput may drop somewhat.  All threads that visit the table
@@ -99,7 +91,7 @@ import org.jctools.util.RangeUtil;
 
 public class NonBlockingHashMapLong<TypeV>
   extends AbstractMap<Long,TypeV>
-  implements ConcurrentMap<Long,TypeV>, Serializable {
+  implements ConcurrentMap<Long,TypeV>, Cloneable, Serializable {
 
   private static final long serialVersionUID = 1234123412341234124L;
 
@@ -124,18 +116,9 @@ public class NonBlockingHashMapLong<TypeV>
   }
 
   // --- Bits to allow Unsafe CAS'ing of the CHM field
-  private static final long _chm_offset;
-  private static final long _val_1_offset;
-  static {                      // <clinit>
-    Field f;
-    try { f = NonBlockingHashMapLong.class.getDeclaredField("_chm"); }
-    catch( java.lang.NoSuchFieldException e ) { throw new RuntimeException(e); }
-    _chm_offset = UNSAFE.objectFieldOffset(f);
+  private static final long _chm_offset = fieldOffset(NonBlockingHashMapLong.class, "_chm");
+  private static final long _val_1_offset = fieldOffset(NonBlockingHashMapLong.class, "_val_1");
 
-    try { f = NonBlockingHashMapLong.class.getDeclaredField("_val_1"); }
-    catch( java.lang.NoSuchFieldException e ) { throw new RuntimeException(e); }
-    _val_1_offset = UNSAFE.objectFieldOffset(f);
-  }
   private final boolean CAS( final long offset, final Object old, final Object nnn ) {
     return UNSAFE.compareAndSwapObject(this, offset, old, nnn );
   }
@@ -212,6 +195,16 @@ public class NonBlockingHashMapLong<TypeV>
       print_impl(i,K,V);
   }
 
+  // Count of reprobes
+  private transient ConcurrentAutoTable _reprobes = new ConcurrentAutoTable();
+  /** Get and clear the current count of reprobes.  Reprobes happen on key
+   *  collisions, and a high reprobe rate may indicate a poor hash function or
+   *  weaknesses in the table resizing function.
+   *  @return the count of reprobes since the last call to {@link #reprobes}
+   *  or since the table was created.   */
+  public long reprobes() { long r = _reprobes.get(); _reprobes = new ConcurrentAutoTable(); return r; }
+
+
   // --- reprobe_limit -----------------------------------------------------
   // Heuristic to decide if we have reprobed toooo many times.  Running over
   // the reprobe limit on a 'get' call acts as a 'miss'; on a 'put' call it
@@ -318,6 +311,7 @@ public class NonBlockingHashMapLong<TypeV>
     return putIfMatch( key, newValue, oldValue ) == oldValue;
   }
 
+  @SuppressWarnings("unchecked")
   private TypeV putIfMatch( long key, Object newVal, Object oldVal ) {
     if (oldVal == null || newVal == null)  throw new NullPointerException();
     if( key == NO_KEY ) {
@@ -343,6 +337,11 @@ public class NonBlockingHashMapLong<TypeV>
     while( !CAS(_chm_offset,_chm,newchm) ) { /*Spin until the clear works*/}
     CAS(_val_1_offset,_val_1,TOMBSTONE);
   }
+  // Non-atomic clear, preserving existing large arrays
+  public void clear(boolean large) {         // Smack a new empty table down
+    _chm.clear();
+    CAS(_val_1_offset,_val_1,TOMBSTONE);
+  }
 
   /** Returns <tt>true</tt> if this Map maps one or more keys to the specified
    *  value.  <em>Note</em>: This method requires a full internal traversal of the
@@ -368,6 +367,7 @@ public class NonBlockingHashMapLong<TypeV>
    *  most one such mapping.)
    * @throws NullPointerException if the specified key is null */
   // Never returns a Prime nor a Tombstone.
+  @SuppressWarnings("unchecked")
   public final TypeV get( long key ) {
     if( key == NO_KEY ) {
       final Object V = _val_1;
@@ -412,6 +412,17 @@ public class NonBlockingHashMapLong<TypeV>
     topchm.help_copy_impl(false);
   }
 
+  // --- hash ----------------------------------------------------------------
+  // Helper function to spread lousy hashCodes Throws NPE for null Key, on
+  // purpose - as the first place to conveniently toss the required NPE for a
+  // null Key.
+  private static final int hash(long h) {
+    h ^= (h>>>20) ^ (h>>>12);
+    h ^= (h>>> 7) ^ (h>>> 4);
+    h += h<<7; // smear low bits up high, for hashcodes that only differ by 1
+    return (int)h;
+  }
+
 
   // --- CHM -----------------------------------------------------------------
   // The control structure for the NonBlockingHashMapLong
@@ -420,7 +431,7 @@ public class NonBlockingHashMapLong<TypeV>
     final NonBlockingHashMapLong _nbhml;
 
     // Size in active K,V pairs
-    private final ConcurrentAutoTable _size;
+    private ConcurrentAutoTable _size;
     public int size () { return (int)_size.get(); }
 
     // ---
@@ -433,7 +444,7 @@ public class NonBlockingHashMapLong<TypeV>
     // keys) then we need a larger table to cut down on the churn.
 
     // Count of used slots, to tell when table is full of dead unusable slots
-    private final ConcurrentAutoTable _slots;
+    private ConcurrentAutoTable _slots;
     public int slots() { return (int)_slots.get(); }
 
     // ---
@@ -486,6 +497,13 @@ public class NonBlockingHashMapLong<TypeV>
       _keys = new long  [1<<logsize];
       _vals = new Object[1<<logsize];
     }
+    // Non-atomic clear
+    void clear() {
+      _size = new ConcurrentAutoTable();
+      _slots= new ConcurrentAutoTable();
+      Arrays.fill(_keys,0);
+      Arrays.fill(_vals,null);
+    }
 
     // --- print innards
     private void print() {
@@ -518,8 +536,9 @@ public class NonBlockingHashMapLong<TypeV>
     // --- get_impl ----------------------------------------------------------
     // Never returns a Prime nor a Tombstone.
     private Object get_impl ( final long key ) {
+      final int hash = hash(key);
       final int len     = _keys.length;
-      int idx = (int)(key & (len-1)); // First key hash
+      int idx = (hash & (len-1)); // First key hash
 
       // Main spin/reprobe loop, looking for a Key hit
       int reprobe_cnt=0;
@@ -562,11 +581,12 @@ public class NonBlockingHashMapLong<TypeV>
     // Only the path through copy_slot passes in an expected value of null,
     // and putIfMatch only returns a null if passed in an expected null.
     private Object putIfMatch( final long key, final Object putval, final Object expVal ) {
+      final int hash = hash(key);
       assert putval != null;
       assert !(putval instanceof Prime);
       assert !(expVal instanceof Prime);
       final int len      = _keys.length;
-      int idx = (int)(key & (len-1)); // The first key
+      int idx = (hash & (len-1)); // The first key
 
       // ---
       // Key-Claim stanza: spin till we can claim a Key (or force a resizing).
@@ -579,8 +599,8 @@ public class NonBlockingHashMapLong<TypeV>
         if( K == NO_KEY ) {     // Slot is free?
           // Found an empty Key slot - which means this Key has never been in
           // this table.  No need to put a Tombstone - the Key is not here!
-          if( putval == TOMBSTONE ) return putval; // Not-now & never-been in this table
-          if( expVal == MATCH_ANY ) return null;   // Will not match, even after K inserts
+          if( putval == TOMBSTONE ) return TOMBSTONE; // Not-now & never-been in this table
+          if( expVal == MATCH_ANY ) return TOMBSTONE; // Will not match, even after K inserts
           // Claim the zero key-slot
           if( CAS_key(idx, NO_KEY, key) ) { // Claim slot for Key
             _slots.add(1);      // Raise key-slots-used count
@@ -621,61 +641,78 @@ public class NonBlockingHashMapLong<TypeV>
         idx = (idx+1)&(len-1); // Reprobe!
       } // End of spinning till we get a Key slot
 
-      // ---
-      // Found the proper Key slot, now update the matching Value slot.  We
-      // never put a null, so Value slots monotonically move from null to
-      // not-null (deleted Values use Tombstone).  Thus if 'V' is null we
-      // fail this fast cutout and fall into the check for table-full.
-      if( putval == V ) return V; // Fast cutout for no-change
+      while ( true ) {              // Spin till we insert a value
+        // ---
+        // Found the proper Key slot, now update the matching Value slot.  We
+        // never put a null, so Value slots monotonically move from null to
+        // not-null (deleted Values use Tombstone).  Thus if 'V' is null we
+        // fail this fast cutout and fall into the check for table-full.
+        if( putval == V ) return V; // Fast cutout for no-change
 
-      // See if we want to move to a new table (to avoid high average re-probe
-      // counts).  We only check on the initial set of a Value from null to
-      // not-null (i.e., once per key-insert).
-      if( (V == null && tableFull(reprobe_cnt,len)) ||
-          // Or we found a Prime: resize is already in progress.  The resize
-          // call below will do a CAS on _newchm forcing the read.
-          V instanceof Prime) {
-        resize();               // Force the new table copy to start
-        return copy_slot_and_check(idx,expVal).putIfMatch(key,putval,expVal);
-      }
-
-      // ---
-      // We are finally prepared to update the existing table
-      //assert !(V instanceof Prime); // always true, so IDE warnings if uncommented
-
-      // Must match old, and we do not?  Then bail out now.  Note that either V
-      // or expVal might be TOMBSTONE.  Also V can be null, if we've never
-      // inserted a value before.  expVal can be null if we are called from
-      // copy_slot.
-
-      if( expVal != NO_MATCH_OLD && // Do we care about expected-Value at all?
-          V != expVal &&            // No instant match already?
-          (expVal != MATCH_ANY || V == TOMBSTONE || V == null) &&
-          !(V==null && expVal == TOMBSTONE) &&    // Match on null/TOMBSTONE combo
-          (expVal == null || !expVal.equals(V)) ) // Expensive equals check at the last
-        return V;                                 // Do not update!
-
-      // Actually change the Value in the Key,Value pair
-      if( CAS_val(idx, V, putval ) ) {
-        // CAS succeeded - we did the update!
-        // Both normal put's and table-copy calls putIfMatch, but table-copy
-        // does not (effectively) increase the number of live k/v pairs.
-        if( expVal != null ) {
-          // Adjust sizes - a striped counter
-          if(  (V == null || V == TOMBSTONE) && putval != TOMBSTONE ) _size.add( 1);
-          if( !(V == null || V == TOMBSTONE) && putval == TOMBSTONE ) _size.add(-1);
+        // See if we want to move to a new table (to avoid high average re-probe
+        // counts).  We only check on the initial set of a Value from null to
+        // not-null (i.e., once per key-insert).
+        if( (V == null && tableFull(reprobe_cnt,len)) ||
+            // Or we found a Prime: resize is already in progress.  The resize
+            // call below will do a CAS on _newchm forcing the read.
+            V instanceof Prime) {
+          resize();               // Force the new table copy to start
+          return copy_slot_and_check(idx,expVal).putIfMatch(key,putval,expVal);
         }
-      } else {                  // Else CAS failed
+
+        // ---
+        // We are finally prepared to update the existing table
+        //assert !(V instanceof Prime); // always true, so IDE warnings if uncommented
+
+        // Must match old, and we do not?  Then bail out now.  Note that either V
+        // or expVal might be TOMBSTONE.  Also V can be null, if we've never
+        // inserted a value before.  expVal can be null if we are called from
+        // copy_slot.
+
+        if( expVal != NO_MATCH_OLD && // Do we care about expected-Value at all?
+            V != expVal &&            // No instant match already?
+            (expVal != MATCH_ANY || V == TOMBSTONE || V == null) &&
+            !(V==null && expVal == TOMBSTONE) &&    // Match on null/TOMBSTONE combo
+            (expVal == null || !expVal.equals(V)) ) // Expensive equals check at the last
+          return (V==null) ? TOMBSTONE : V;         // Do not update!
+
+        // Actually change the Value in the Key,Value pair
+        if( CAS_val(idx, V, putval ) ) break;
+
+        // CAS failed
+        // Because we have no witness, we do not know why it failed.
+        // Indeed, by the time we look again the value under test might have flipped
+        // a thousand times and now be the expected value (despite the CAS failing).
+        // Check for the never-succeed condition of a Prime value and jump to any
+        // nested table, or else just re-run.
+
+        // We would not need this load at all if CAS returned the value on which
+        // the CAS failed (AKA witness). The new CAS semantics are supported via
+        // VarHandle in JDK9.
         V = _vals[idx];         // Get new value
+
         // If a Prime'd value got installed, we need to re-run the put on the
         // new table.  Otherwise we lost the CAS to another racing put.
         // Simply retry from the start.
         if( V instanceof Prime )
           return copy_slot_and_check(idx,expVal).putIfMatch(key,putval,expVal);
+
+        // Simply retry from the start.
+        // NOTE: need the fence, since otherwise '_vals[idx]' load could be hoisted
+        // out of loop.
+        int dummy = DUMMY_VOLATILE;
       }
-      // Win or lose the CAS, we are done.  If we won then we know the update
-      // happened as expected.  If we lost, it means "we won but another thread
-      // immediately stomped our update with no chance of a reader reading".
+
+      // CAS succeeded - we did the update!
+      // Both normal put's and table-copy calls putIfMatch, but table-copy
+      // does not (effectively) increase the number of live k/v pairs.
+      if( expVal != null ) {
+        // Adjust sizes - a striped counter
+        if(  (V == null || V == TOMBSTONE) && putval != TOMBSTONE ) _size.add( 1);
+        if( !(V == null || V == TOMBSTONE) && putval == TOMBSTONE ) _size.add(-1);
+      }
+
+      // We won; we know the update happened as expected.
       return (V==null && expVal!=null) ? TOMBSTONE : V;
     }
 
@@ -688,7 +725,7 @@ public class NonBlockingHashMapLong<TypeV>
     // current table, while a 'get' has decided the same key cannot be in this
     // table because of too many reprobes.  The invariant is:
     //   slots.estimate_sum >= max_reprobe_cnt >= reprobe_limit(len)
-    private final boolean tableFull( int reprobe_cnt, int len ) {
+    private boolean tableFull( int reprobe_cnt, int len ) {
       return
         // Do the cheap check first: we allow some number of reprobes always
         reprobe_cnt >= REPROBE_LIMIT &&
@@ -703,7 +740,7 @@ public class NonBlockingHashMapLong<TypeV>
     // Since this routine has a fast cutout for copy-already-started, callers
     // MUST 'help_copy' lest we have a path which forever runs through
     // 'resize' only to discover a copy-in-progress which never progresses.
-    private final CHM resize() {
+    private CHM resize() {
       // Check for resize already in progress, probably triggered by another thread
       CHM newchm = _newchm;     // VOLATILE READ
       if( newchm != null )      // See if resize is already in progress
@@ -735,7 +772,7 @@ public class NonBlockingHashMapLong<TypeV>
       // the table needs a steady state of rare same-size resize
       // operations to clean out the dead keys.
       long tm = System.currentTimeMillis();
-      if( newsz <= oldlen && // New table would shrink or hold steady?
+      if( newsz <= oldlen &&    // New table would shrink or hold steady?
           tm <= _nbhml._last_resize_milli+10000)  // Recent resize (less than 10 sec ago)
         newsz = oldlen<<1;      // Double the existing size
 
@@ -823,7 +860,7 @@ public class NonBlockingHashMapLong<TypeV>
     // Help along an existing resize operation.  We hope its the top-level
     // copy (it was when we started) but this CHM might have been promoted out
     // of the top position.
-    private final void help_copy_impl( final boolean copy_all ) {
+    private void help_copy_impl( final boolean copy_all ) {
       final CHM newchm = _newchm;
       assert newchm != null;    // Already checked by caller
       int oldlen = _keys.length; // Total amount to copy
@@ -845,7 +882,8 @@ public class NonBlockingHashMapLong<TypeV>
         // thread counts trying to copy the table often 'panic'.
         if( panic_start == -1 ) { // No panic?
           copyidx = (int)_copyIdx;
-          while( !_copyIdxUpdater.compareAndSet(this,copyidx,copyidx+MIN_COPY_WORK) )
+          while( copyidx < (oldlen<<1) && // 'panic' check
+                 !_copyIdxUpdater.compareAndSet(this,copyidx,copyidx+MIN_COPY_WORK) )
             copyidx = (int)_copyIdx;     // Re-read
           if( !(copyidx < (oldlen<<1)) ) // Panic!
             panic_start = copyidx;       // Record where we started to panic-copy
@@ -897,23 +935,25 @@ public class NonBlockingHashMapLong<TypeV>
     }
 
     // --- copy_check_and_promote --------------------------------------------
-     private final void copy_check_and_promote( int workdone ) {
+    private void copy_check_and_promote( int workdone ) {
       int oldlen = _keys.length;
       // We made a slot unusable and so did some of the needed copy work
       long copyDone = _copyDone;
-      assert (copyDone+workdone) <= oldlen;
+      long nowDone = copyDone+workdone;
+      assert nowDone <= oldlen;
       if( workdone > 0 ) {
-        while( !_copyDoneUpdater.compareAndSet(this,copyDone,copyDone+workdone) ) {
-          copyDone = _copyDone; // Reload, retry
-          assert (copyDone+workdone) <= oldlen; 
+        while( !_copyDoneUpdater.compareAndSet(this,copyDone,nowDone) ) {
+          copyDone = _copyDone;   // Reload, retry
+          nowDone = copyDone+workdone;
+          assert nowDone <= oldlen;
         }
       }
 
       // Check for copy being ALL done, and promote.  Note that we might have
       // nested in-progress copies and manage to finish a nested copy before
       // finishing the top-level copy.  We only promote top-level copies.
-      if( copyDone+workdone == oldlen && // Ready to promote this table?
-          _nbhml._chm == this &&         // Looking at the top-level table?
+      if( nowDone == oldlen &&   // Ready to promote this table?
+          _nbhml._chm == this && // Looking at the top-level table?
           // Attempt to promote
           _nbhml.CAS(_chm_offset,this,_newchm) ) {
         _nbhml._last_resize_milli = System.currentTimeMillis();  // Record resize time for next check
@@ -970,7 +1010,7 @@ public class NonBlockingHashMapLong<TypeV>
       // appears in the old table.
       Object old_unboxed = ((Prime)oldval)._V;
       assert old_unboxed != TOMBSTONE;
-      _newchm.putIfMatch(key, old_unboxed, null);
+      boolean copied_into_new = (_newchm.putIfMatch(key, old_unboxed, null) == null);
 
       // ---
       // Finally, now that any old value is exposed in the new table, we can
@@ -980,7 +1020,7 @@ public class NonBlockingHashMapLong<TypeV>
       while( oldval != TOMBPRIME && !CAS_val(idx,oldval,TOMBPRIME) )
         oldval = _vals[idx];
 
-      return oldval != TOMBPRIME; // True if we slammed the TOMBPRIME down
+      return copied_into_new;
     } // end copy_slot
   } // End of CHM
 
@@ -1039,11 +1079,21 @@ public class NonBlockingHashMapLong<TypeV>
       }                         // Else keep scanning
       return _prevV;            // Return current value.
     }
-    public void remove() {
+
+    public void removeKey() {
       if( _prevV == null ) throw new IllegalStateException();
-      _sschm.putIfMatch( _prevK, TOMBSTONE, _prevV );
+      NonBlockingHashMapLong.this.putIfMatch( _prevK, TOMBSTONE, NO_MATCH_OLD);
       _prevV = null;
     }
+
+    @Override
+    public void remove() {
+      // NOTE: it would seem logical that value removal will semantically mean
+      // removing the matching value for the mapping <k,v>, but the JDK always
+      // removes by key, even when the value has changed.
+      removeKey();
+    }
+
     public TypeV nextElement() { return next(); }
     public boolean hasMoreElements() { return hasNext(); }
   }
@@ -1085,7 +1135,7 @@ public class NonBlockingHashMapLong<TypeV>
     /** A new IteratorLong */
     public IteratorLong() { _ss = new SnapshotV(); }
     /** Remove last key returned by {@link #next} or {@link #nextLong}. */
-    public void remove() { _ss.remove(); }
+    public void remove() { _ss.removeKey(); }
     /** <strong>Auto-box</strong> and return the next key. */
     public Long next    () { _ss.next(); return _ss._prevK; }
     /** Return the next key as a primitive {@code long}. */
@@ -1128,6 +1178,7 @@ public class NonBlockingHashMapLong<TypeV>
   }
 
   /** Keys as a long array.  Array may be zero-padded if keys are concurrently deleted. */
+  @SuppressWarnings("unchecked")
   public long[] keySetLong() {
     long[] dom = new long[size()];
     IteratorLong i=(IteratorLong)keySet().iterator();
@@ -1151,7 +1202,12 @@ public class NonBlockingHashMapLong<TypeV>
   private class SnapshotE implements Iterator<Map.Entry<Long,TypeV>> {
     final SnapshotV _ss;
     public SnapshotE() { _ss = new SnapshotV(); }
-    public void remove() { _ss.remove(); }
+    public void remove() {
+      // NOTE: it would seem logical that entry removal will semantically mean
+      // removing the matching pair <k,v>, but the JDK always removes by key,
+      // even when the value has changed.
+      _ss.removeKey();
+    }
     public Map.Entry<Long,TypeV> next() { _ss.next(); return new NBHMLEntry(_ss._prevK,_ss._prevV); }
     public boolean hasNext() { return _ss.hasNext(); }
   }
@@ -1175,7 +1231,7 @@ public class NonBlockingHashMapLong<TypeV>
    *  iteration.  The {@link org.jctools.maps.NonBlockingHashMap}
    *  does not normally create or using {@link java.util.Map.Entry} objects so
    *  they will be created soley to support this iteration.  Iterating using
-   *  {@link #keySet} or {@link #values} will be more efficient.  In addition,
+   *  {@link Map#keySet} or {@link Map#values} will be more efficient.  In addition,
    *  this version requires <strong>auto-boxing</strong> the keys.
    */
   public Set<Map.Entry<Long,TypeV>> entrySet() {
@@ -1191,7 +1247,7 @@ public class NonBlockingHashMapLong<TypeV>
         if (!(o instanceof Map.Entry)) return false;
         final Map.Entry<?,?> e = (Map.Entry<?,?>)o;
         TypeV v = get(e.getKey());
-        return v.equals(e.getValue());
+        return v != null && v.equals(e.getValue());
       }
       public Iterator<Map.Entry<Long,TypeV>> iterator() { return new SnapshotE(); }
     };
@@ -1211,7 +1267,8 @@ public class NonBlockingHashMapLong<TypeV>
   }
 
   // --- readObject --------------------------------------------------------
-  // Read a CHM from a stream
+  // Read a NBHML from a stream
+  @SuppressWarnings("unchecked")
   private void readObject(java.io.ObjectInputStream s) throws IOException, ClassNotFoundException  {
     s.defaultReadObject();      // Read nothing
     initialize(MIN_SIZE);
@@ -1223,4 +1280,34 @@ public class NonBlockingHashMapLong<TypeV>
     }
   }
 
+  /**
+   * Creates a shallow copy of this hashtable. All the structure of the
+   * hashtable itself is copied, but the keys and values are not cloned.
+   * This is a relatively expensive operation.
+   *
+   * @return  a clone of the hashtable.
+   */
+  @SuppressWarnings("unchecked")
+  @Override
+  public NonBlockingHashMapLong<TypeV> clone() {
+    try {
+      // Must clone, to get the class right; NBHML might have been
+      // extended so it would be wrong to just make a new NBHML.
+      NonBlockingHashMapLong<TypeV> t = (NonBlockingHashMapLong<TypeV>) super.clone();
+      // But I don't have an atomic clone operation - the underlying _kvs
+      // structure is undergoing rapid change.  If I just clone the _kvs
+      // field, the CHM in _kvs[0] won't be in sync.
+      //
+      // Wipe out the cloned array (it was shallow anyways).
+      t.clear();
+      // Now copy sanely
+      for( long K : keySetLong() )
+        t.put(K,get(K));
+      return t;
+    } catch (CloneNotSupportedException e) {
+      // this shouldn't happen, since we are Cloneable
+      throw new InternalError();
+    }
+  }
+
 }  // End NonBlockingHashMapLong class
