@@ -13,41 +13,60 @@
  */
 package org.jctools.queues.atomic;
 
+import org.jctools.util.InternalAPI;
+
 import java.util.concurrent.atomic.AtomicReference;
 
-public final class LinkedQueueAtomicNode<E> extends AtomicReference<LinkedQueueAtomicNode<E>> {
+@InternalAPI
+public final class LinkedQueueAtomicNode<E> extends AtomicReference<LinkedQueueAtomicNode<E>>
+{
     /** */
     private static final long serialVersionUID = 2404266111789071508L;
     private E value;
-    LinkedQueueAtomicNode() {
+
+    public LinkedQueueAtomicNode()
+    {
     }
-    LinkedQueueAtomicNode(E val) {
+
+    public LinkedQueueAtomicNode(E val)
+    {
         spValue(val);
     }
+
     /**
      * Gets the current value and nulls out the reference to it from this node.
-     * 
+     *
      * @return value
      */
-    public E getAndNullValue() {
+    public E getAndNullValue()
+    {
         E temp = lpValue();
         spValue(null);
         return temp;
     }
 
-    public E lpValue() {
+    public E lpValue()
+    {
         return value;
     }
 
-    public void spValue(E newValue) {
+    public void spValue(E newValue)
+    {
         value = newValue;
     }
 
-    public void soNext(LinkedQueueAtomicNode<E> n) {
+    public void soNext(LinkedQueueAtomicNode<E> n)
+    {
         lazySet(n);
     }
 
-    public LinkedQueueAtomicNode<E> lvNext() {
+    public void spNext(LinkedQueueAtomicNode<E> n)
+    {
+        lazySet(n);
+    }
+
+    public LinkedQueueAtomicNode<E> lvNext()
+    {
         return get();
     }
 }
