@@ -16,12 +16,10 @@ package org.jctools.util;
 /**
  * Power of 2 utility functions.
  */
+@InternalAPI
 public final class Pow2 {
     public static final int MAX_POW2 = 1 << 30;
 
-    private Pow2() {
-    }
-
     /**
      * @param value from which next positive power of two will be found.
      * @return the next positive power of 2, this value if it is a power of 2. Negative values are mapped to 1.
